import java.io.*;
import java.text.*;
import java.util.*;

class FileDemo {
	public static void main(String[] args) {
		byte[] arr = {1, 2, 3};

		// Verzeichnis ggf. erstellen
		String ordner = "U:/Eigene Dateien/Java/FileDemo/AusDir";
		File dir = new File(ordner);
		if (dir.exists()) {
			if (dir.isDirectory())
				System.out.println("Das Verzeichnis " + ordner + " existiert bereits.");
			else {
				System.out.println(ordner + " existiert, ist aber kein Verzeichnis.");
				System.exit(1);
			}
		} else
			if (dir.mkdirs())
				System.out.println("Verzeichnis " + ordner + " erstellt");
			else {
				System.out.println("Verzeichnis " + ordner + " konnte nicht erstellt werden.");
				System.exit(1);
			}

		// Datei explizit erstellen
		String name = ordner + "/Ausgabe.dat";
		File f = new File(name);
		if (!f.exists()) {
			try {
				f.createNewFile();
				System.out.println("Datei " + name + " erstellt");
			} catch (Exception e) {
				System.out.println("Fehler beim Erstellen der Datei " +name);
				System.exit(1);
			}
		}

		// Ausgabedatei öffnen und schreiben
		// (unter Verzicht auf die automatische Ressourcen-Freigabe von Java 7)
		FileOutputStream fos = null;
		boolean error = false;
		try {
			fos = new FileOutputStream(f);
			fos.write(arr);
			System.out.println("Daten geschrieben in Datei " + name);
		} catch (Exception e) {
			System.out.println("Fehler beim Schreiben in Datei " + name);
			error = true;
		} finally {
			try {
				if (fos != null)
					fos.close();
			} catch (Exception x) {};			
		}
		if (error)
			System.exit(1);

		// Über Ausgabedatei informieren
		System.out.println("\nEigenschaften der Datei " + f.getName());
		System.out.println("  Vollst. Pfad:       " + f.getAbsolutePath());
		DateFormat df = DateFormat.getInstance();
		String time = df.format(new Date(f.lastModified()));
		System.out.println("  Letzte Änderung:    " + time);
		System.out.println("  Größe in Bytes:     " + f.length());
		System.out.println("  Schreiben möglich:  " + f.canWrite()+"\n");

		// Attribute der Ausgabedatei ändern
		System.out.println("\nÄnderung von Attributen der Datei " + f.getName());
		Date d = null;
		df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
		try {
			d = df.parse("24.12.2022 16:15:00");
			time = df.format(d);
		} catch (Exception e) {
			System.out.println("Fehler bei der Datumsinterpretation");
			d = new Date();
		}
		System.out.println("  setLastModified() erfolgreich:    " + f.setLastModified(d.getTime()));
		System.out.println("  Letzte Änderung:                  " + time);		
		System.out.println("  setWritable(false) erfolgreich:   " + f.setWritable(false));
		System.out.println("  setWritable(true) erfolgreich:    " + f.setWritable(true) +"\n");

		// Verzeichnisinhalt auflisten
		File curDir = new File(".");
		File[] files = curDir.listFiles();
		System.out.println("Einträge im aktuellen Verzeichnis:");
		for (File fi : files)
			System.out.println(" " + fi.getName());
		files = curDir.listFiles(new FNFilter("iml"));
		System.out.println("\nEinträge im aktuellen Verzeichnis mit Extension .iml:");
		for (File fi : files)
			System.out.println(" " + fi.getName());

		// Nutzbares Speichervolumen ermitteln
		System.out.println("\nNutzbarer Speicher in " + dir.getAbsoluteFile() +
				           ": " + dir.getUsableSpace()/1024/1024 + " MB");
		if(1==1)return;
		// Ausgabedatei umbenennen
		File fn = new File(ordner + "/Rausgabe.dat");
		if (f.renameTo(fn))
			System.out.println("\nDatei " + f.getName() + " umbenannt in " + fn.getName());
		else {
			System.out.println("Fehler beim Umbenennen der Datei " + f.getName());
			fn = f;
		}

		// Ausgabedatei löschen
		if (fn.delete())
			System.out.println("\nDatei " + fn.getName() + " gelöscht");
		else {
			System.out.println("\nFehler beim Löschen der Datei " + fn.getName());
			System.exit(1);
		}

		// Verzeichnisse löschen
		if (dir.delete() && (new File(dir.getParent()).delete())) {
			System.out.println("\nVerzeichnis " + dir.getAbsolutePath() + " gelöscht");
			System.out.println("Verzeichnis " + dir.getParent() + " gelöscht");
		} else
			System.out.println("\nFehler beim Löschen der Ordner");
	}
}