import java.io.*;
import java.nio.file.*;
import java.nio.file.attribute.*;
import java.util.*;

class FilesAndFolders {
   
	static void getSpaceOfStore(FileStore store) throws IOException {
		final long MEGA = 1024*1024;
		long gesamt = store.getTotalSpace()/MEGA;
		long belegt = (store.getTotalSpace()-store.getUsableSpace())/MEGA;
		long frei = store.getUsableSpace()/MEGA;
		System.out.printf("%-20s %12d %12d %12d\n", store, gesamt, belegt, frei);
	}
	
	public static void main(String[] args)  throws IOException {
		byte[] arr = {1, 2, 3};

		Path dir = Paths.get("U:", "Eigene Dateien", "Java", "io");
		
		// Sich später auf ein Testergebnis zu verlassen, ist unsicher (TOCTTOU-Falle).
		if (Files.exists(dir))
			System.out.println(dir+" existiert bereits.");
		else
			if (Files.notExists(dir))
				System.out.println(dir+" existiert noch nicht");
			else
				System.out.println(dir+" hat einen unbekannten Status.");

		// Verzeichnis ggf. erstellen
		try {
			Files.createDirectories(dir);
			System.out.println(dir+" erstellt");
		} catch (FileAlreadyExistsException ae) {
			System.err.println(dir + " existiert, ist aber kein Verzeichnis.");
			System.exit(1);
		}

		// Datei explizit erstellen
		Path file = dir.resolve("Ausgabe.dat");
		try {
			Files.createFile(file);
		} catch (FileAlreadyExistsException ae) {
			System.err.println(file + " existiert bereits.");
		}

		// Daten in die Ausgabedatei schreiben
		Files.write(file,  arr);
		System.out.println("Daten geschrieben in Datei "+file);

		// Über ein Dateisystemobjekt informieren
		System.out.println("\nEigenschaften von    " + file);
//		BasicFileAttributes attr = Files.readAttributes(file, BasicFileAttributes.class);
//		BasicFileAttributes attr = Files.readAttributes(name, BasicFileAttributes.class, LinkOption.NOFOLLOW_LINKS);
//		System.out.println(" Größe in Bytes:     " + attr.size());
		System.out.println(" Größe in Bytes:     " + Files.size(file));
		System.out.println(" Letzte Änderung:    " + Files.getLastModifiedTime(file));
		System.out.println(" Datei:              " + Files.isRegularFile(file));
		System.out.println(" Verzeichnis:        " + Files.isDirectory(file));
		System.out.println(" Schreiben möglich:  " + Files.isWritable(file));
		System.out.println(" Symb. Link:         " + Files.isSymbolicLink(file));

		// Datum der letzten Modifikation ändern
		System.out.println("\nDatum der letzten Modifikation setzen\n vorher:    " + Files.getLastModifiedTime(file));
		Calendar cal = Calendar.getInstance();
		cal.set(1980, Calendar.APRIL, 1, 0, 0, 0);
		Files.setLastModifiedTime(file, FileTime.fromMillis(cal.getTimeInMillis()));
		System.out.println(" nachher:   " + Files.getLastModifiedTime(file));

		// Datei kopieren
		Files.copy(file, dir.resolve("Kopie.dat"), StandardCopyOption.REPLACE_EXISTING);

		// Ordner kopieren
		Path targetDir = dir.resolveSibling("io2");
		try {
			Files.copy(dir, targetDir);
		} catch (FileAlreadyExistsException ae) {
			System.out.printf("\n%s existiert bereits.\n", targetDir);
		}

		// Datei umbenennen
		Files.move(file, file.resolveSibling("Umbenannt.dat"), StandardCopyOption.REPLACE_EXISTING);
		System.out.println("\nDatei " + file + " umbenannt");

		// Datei verschieben
		Files.move(dir.resolve("Umbenannt.dat"), dir.getParent().resolve("Verschoben.dat"),
				StandardCopyOption.REPLACE_EXISTING, StandardCopyOption.ATOMIC_MOVE);
		System.out.println("\nDatei " + dir.resolve("Umbenannt.dat") + " verschoben");

		// Verzeichnisinhalte auflisten
		System.out.println("\nDateien im Verzeichnis " + dir);
		try (DirectoryStream<Path> ds = Files.newDirectoryStream(dir)) {
			for (Path path: ds)
				System.out.println(" " + path.getFileName());
		}

		// Dateien löschen
		try (DirectoryStream<Path> ds = Files.newDirectoryStream(dir)) {
			System.out.println("\nDateien löschen:");
			for (Path path: ds) {
				Files.delete(path);
				System.out.println(" " + path.getFileName());
			}
		} catch (IOException ioe) {
			//  Gelegentlich scheitert das Löschen bei einem Relikt vom obigen Umbenennen:
			//	java.nio.file.NoSuchFileException: U:\Eigene Dateien\Java\io\UMBENANNT.DAT.tmp
			//  at sun.nio.fs.WindowsException.translateToIOException(WindowsException.java:79)
			//	at sun.nio.fs.WindowsException.rethrowAsIOException(WindowsException.java:97)
			//	at sun.nio.fs.WindowsException.rethrowAsIOException(WindowsException.java:102)
			//	at sun.nio.fs.WindowsFileSystemProvider.implDelete(WindowsFileSystemProvider.java:269)
			//	at sun.nio.fs.AbstractFileSystemProvider.delete(AbstractFileSystemProvider.java:103)
			//	at java.nio.file.Files.delete(Files.java:1126)
			//	at FilesAndFolders.main(FilesAndFolders.java:101)
			ioe.printStackTrace();
		}

		// Verzeichnisse löschen
		try {
			Files.delete(dir);
			Files.delete(dir.resolveSibling("io2"));
			System.out.println("\nVerzeichnisse gelöscht");
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}

		// Nutzbares Speichervolumen ermitteln
        System.out.printf("\n%-20s %12s %12s %12s\n", "Laufwerk", "Gesamt (MB)", "Belegt (MB)", "Frei (MB)");
        FileSystem fs = FileSystems.getDefault();
		try {
			for (FileStore store: fs.getFileStores())
				getSpaceOfStore(store);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
	}
}